
package Perl6::Role::Method;

use strict;
use warnings;

use base 'Perl6::Instance::Method';

1;

__END__

=pod

=head1 NAME

Perl6::Role::Method - Role Methods in the Perl 6 Meta Model

=head1 DESCRIPTION

=head1 SUPERCLASS

NOTE: I am not sure if this is correct. It is possible Roles might
inject class methods as well. 

=over 4

=item I<Perl6::Instance::Method>

=back

=head1 AUTHOR

Stevan Little E<lt>stevan@iinteractive.comE<gt>

=cut
