#=der x Operator
# x Wiederholt in Perl weiterhin Strings:
$a = "a" x 3  # "aaa"

#=der xx Operator
# neu in Perl 6 ist der xx Operator
# so wie x Strings wiederholt, wiederholt xx listen:
@a = (1, 2) xx 2 # (1, 2, 1, 2)

