#=Skalare (Zahlen, Strings, Objekte, ...)
Perl 5:  my $name="Jens"; print "Mein Name ist $name.";
Perl 6:  my $name="Jens"; print "Mein Name ist $name.";
#=Arrays
Perl 5: my @array = (1,2,3); print "$a[0]";
Perl 6: my @array = (1,2,3); print "@a[0]";
#=Hashes
Perl 5: my %hash = {a=>'b'}; print "nach a kommt $hash{a}";
Perl 6: my %hash = {a=>'b'}; print "nach a kommt %hash{a}";
