
package Perl6::Role::Attribute;

use strict;
use warnings;

use base 'Perl6::Instance::Attribute';

1;

__END__

=pod

=head1 NAME

Perl6::Role::Attribute - Role Attributes in the Perl 6 Meta Model

=head1 DESCRIPTION

=head1 SUPERCLASS

NOTE: As with Perl6::Role::Method, this may not be correct and Roles
might be able to inject Class level attributes as well.

=over 4

=item I<Perl6::Instance::Attribute>

=back

=head1 AUTHOR

Stevan Little E<lt>stevan@iinteractive.comE<gt>

=cut
