#=Variablen
werden weiterhin mit den "magischen Zeichen" (sigils)
$, %, @ gekennzeichnet.
#=Kontextunabhängigkeit
Da die kontextabhängige Sigil-Wahl in Perl 5 Newcommern oft
Probleme bereitet, wird sie in Perl 6 nur noch vom Datentyp
und nicht mehr vom Kontext abhängig sein.
