
role NameSpace--Perl6;

has Str $.name;

=head1 NAME

NameSpace - package namespaces

=head1 SYNOPSIS

 class Role does NameSpace;

=head1 DESCRIPTION

This type is not used directly; however, it adds the "name" property
when included as a role.

=cut

