#=Beispielverknüpfungen
$a != $b;
$a ~~ $b;    # smart-match, mehr dazu gleich
#=Verknüpfungen in Perl 5:
$a < $b && $b <= $c;
$a > $b && $b >= $c;
#=Perl 6: Verkettung ist möglich:
$a < $b <= $c;
$a > $b >= $c;
